@@ -11,6 +11,21 @@ Geo::WebService::Elevation::USGS - Elevation queries against USGS web services.
    $eq->getElevation(38.898748, -77.037684)->{Elevation},
    " feet above sea level.\n";
 
+=head1 NOTICE
+
+This module has been converted to use HTTP Post instead of SOAP as a
+transport, due to the slowness of SOAP::Lite in fixing test errors.
+
+In the transition version, 0.007_01, the C<'transport'> attribute was
+made available to select which transport was used. That module also
+documented that this attribute would be revoked in the next production
+version unless I got feedback to the contrary.
+
+It is now time for the production release, and I have received no
+feedback (either asking for the retention of SOAP::Lite or otherwise).
+So with this release, support for SOAP as a transport is revoked, as is
+the C<'transport'> attribute.
+
 =head1 DESCRIPTION
 
 This module executes elevation queries against the United States
@@ -88,10 +103,12 @@ use strict;
 use warnings;
 
 use Carp;
+use HTTP::Request::Common;
+use LWP::UserAgent;
 use Scalar::Util 1.10 qw{ blessed looks_like_number };
-use SOAP::Lite;
+use XML::Parser;
 
-our $VERSION = '0.007';
+our $VERSION = '0.008';
 
 use constant BEST_DATA_SET => -1;
 
@@ -147,7 +164,7 @@ sub new {
 	error	=> undef,
 	places	=> undef,
 	proxy	=>
-	    'http://gisdata.usgs.gov/xmlwebservices2/elevation_service.asmx',
+	    'http://gisdata.usgs.gov/XMLWebServices2/Elevation_Service.asmx',
 	retry	=> 0,
 	retry_hook => sub {},
 	source	=> BEST_DATA_SET,
@@ -381,7 +398,6 @@ returned. Note that this may result in an empty array.
 
 sub getAllElevations {
     my ($self, $lat, $lon, $valid) = _latlon( @_ );
-    my $soap = $self->_soapdish();
     my $retry_limit = $self->get( 'retry' );
     my $retry = 0;
 
@@ -393,25 +409,11 @@ sub getAllElevations {
 
 	my $raw;
 	eval {
-	    if ( exists $self->{_hack_result} ) {
-		$raw = delete $self->{_hack_result}
-
-	    } else {
-
-		local $SOAP::Constants::DO_NOT_USE_CHARSET = 1;
-
-		$raw = $soap->call(
-		    SOAP::Data->name ('getAllElevations')->attr (
-			{xmlns => $self->{default_ns}}) =>
-		    SOAP::Data->new(name => 'X_Value', type => 'string',
-			value => $lon),
-		    SOAP::Data->new(name => 'Y_Value', type => 'string',
-			value => $lat),
-		    SOAP::Data->new(name => 'Elevation_Units', type => 'string',
-			value => $self->{units}),
-		);
-	    }
-
+	    $raw = $self->_request( getAllElevations =>
+		X_Value	=> $lon,
+		Y_Value	=> $lat,
+		Elevation_Units	=> $self->{units},
+	    );
 	    1;
 	} or do {
 	    $self->_error( $@ );
@@ -530,7 +532,6 @@ my $bad_extent_re = _make_matcher(
 sub getElevation {
     my ($self, $lat, $lon, $source, $only) = _latlon( @_ );
     defined $source or $source = BEST_DATA_SET;
-    my $soap = $self->_soapdish();
     my $retry_limit = $self->get( 'retry' );
     my $retry = 0;
 
@@ -542,40 +543,19 @@ sub getElevation {
 
 	my $rslt;
 	eval {
-
-	    if ( exists $self->{_hack_result} ) {
-		$rslt = delete $self->{_hack_result};
-	    } else {
-
-		local $SOAP::Constants::DO_NOT_USE_CHARSET = 1;
-
-		$rslt = $soap->call(SOAP::Data->name ('getElevation')->attr (
-			{xmlns => $self->{default_ns}}) =>
-		    SOAP::Data->new(name => 'X_Value', type => 'string',
-			value => $lon),
-		    SOAP::Data->new(name => 'Y_Value', type => 'string',
-			value => $lat),
-		    SOAP::Data->new(name => 'Source_Layer', type => 'string', 
-			value => $source),
-		    SOAP::Data->new(name => 'Elevation_Units', type => 'string',
-			value => $self->{units}),
-		    SOAP::Data->new(name => 'Elevation_Only', type => 'string',
-			value => $only ? 'true' : 'false'),
-		);
-	    }
+	    $rslt = $self->_request( getElevation =>
+		X_Value	=> $lon,
+		Y_Value	=> $lat,
+		Source_Layer	=> $source,
+		Elevation_Units	=> $self->{units},
+		Elevation_Only	=> $only ? 'true' : 'false',
+	    );
 	    1;
 	} or do {
 	    $self->_error( $@ );
 	    next;
 	};
 
-	if ( _instance( $rslt, 'SOAP::SOM' ) && $rslt->fault &&
-		$rslt->faultstring =~ m/ $bad_extent_re /smxo ) {
-	    if (my $hash = $self->_get_bad_extent_hash($source)) {
-		return $hash;
-	    }
-	}
-
 	my $info = $self->_digest( $rslt, $source ) or next;
 	return $info;
 
@@ -625,7 +605,11 @@ result in an exception being thrown.
 
 {
 
-    my %clean_soapdish;	# Attributes that are used in _soapdish();
+    # Changes in these values require re-instantiating the transport
+    # object. Or at least, they may do, under the following assumptions:
+    # SOAP: default_ns, proxy, timeout;
+    # HTTP_Post: timeout.
+    my %clean_transport_object = map { $_ => 1 } qw{ default_ns proxy timeout };
 
     sub set {	## no critic (ProhibitAmbiguousNames)
 	my ($self, @args) = @_;
@@ -638,17 +622,12 @@ result in an exception being thrown.
 		or croak "Attribute '$name' is read-only";
 	    my $holder = $access_type{$name}->( $self, $name );
 	    $mutator{$name}->( $holder, $name, $val );
-	    $clean ||= $clean_soapdish{$name};
+	    $clean ||= $clean_transport_object{$name};
 	}
-	$clean and delete $self->{_soapdish};
+	$clean and delete $self->{_transport_object};
 	return $self;
     }
 
-    sub _set_clean_soapdish {
-	%clean_soapdish = map { $_ => 1 } @_;
-	return;
-    }
-
 }
 
 sub _set_hook {
@@ -742,14 +721,10 @@ my $no_elevation_value_re = _make_matcher(
 
 sub _digest {
     my ($self, $rslt, $source) = @_;
-    if ( _instance( $rslt, 'SOAP::SOM' ) ) {
-	if ($rslt->fault) {
-	    return $self->_error($rslt->faultstring);
-	} else {
-	    $rslt = $rslt->result;
-	}
-    }
-    $self->{trace} and SOAP::Trace->import('-all');
+
+##  use YAML;
+##  warn 'Debug - response content is ', ref $rslt ? Dump( $rslt ) : $rslt;
+
     my $round;
     {
 	my $prec = $self->{places};
@@ -787,7 +762,7 @@ sub _digest {
 		"$rslt Source = '$source'" : $rslt;
 	}
     } else {
-	$error = "No data found in SOAP result";
+	$error = "No data found in query result";
     }
 
     $error
@@ -1021,35 +996,114 @@ sub _make_matcher {	## no critic (RequireArgUnpacking)
 
 sub _normalize_id {return uc $_[0]}
 
-#	$soap_object = _soapdish ()
-
-#	Manufacture a SOAP::Lite object from the object's attributes.
-#	We need this because SOAP::Lite has changed significantly since
-#	the 2002 version that as of 2006 was still bundled with
-#	ActivePerl.
+#	$rslt = $self->_request( $service, %args );
 #
-#	We also set the error attribute and $@ to undef in preparation
-#	for a query.
+#	This private method requests data from the USGS' web service.
+#	The $service argument is the name of the specific request (
+#	'getAllElevations' or 'getElevation' ), and the %args are the
+#	arguments for the specific request. The return is a reference to
+#	a hash containing the parsed XML returned from the USGS.
+#
+#	During the transition from SOAP to HTTP Post as a transport
+#	(i.e. version 0.007_01) this dispatched based on the now-revoked
+#	'transport' attribute. But that attribute is revoked, and this
+#	method is now a trampoline to the HTTP Post logic.
 
-_set_clean_soapdish( qw{ default_ns proxy timeout } );
+sub _request {
+    goto &_request_HTTP_Post;
+}
 
-sub _soapdish {
-    my $self = shift;
-    $self->{trace} and SOAP::Trace->import ('all');
-    $self->{_soapdish} and return $self->{_soapdish};
-    my $conn = '';	# Other possibility is '#'.
-    my $act = sub {join $conn, @_};
-    my $soap = SOAP::Lite->can ('default_ns') ?
-	SOAP::Lite
-	    ->default_ns ($self->{default_ns})
-	    ->on_action ($act)
-	    ->proxy ($self->{proxy}, timeout => $self->{timeout}) :
-	SOAP::Lite
-	    ->envprefix ('soap')
-	    ->on_action ($act)
-	    ->uri ($self->{default_ns})
-	    ->proxy ($self->{proxy}, timeout => $self->{timeout});
-    return ( $self->{_soapdish} = $soap );
+#	$rslt = $self->_request_HTTP_Post( $service, %args );
+#
+#	This private method requests data from the USGS' web service
+#	using an HTTP Post request. It has the same signature as
+#	_request().
+#
+#	THIS METHOD SHOULD ONLY BE CALLED BY _request().
+
+sub _request_HTTP_Post {
+    my ( $self, $service, %arg ) = @_;
+
+    my $rslt;
+
+    if ( exists $self->{_hack_result} ) {
+
+	$rslt = delete $self->{_hack_result};
+	'CODE' eq ref $rslt
+	    and $rslt = $rslt->( $self, $service, %arg );
+
+    } else {
+
+	my $ua = $self->{_transport_object} ||=
+	    LWP::UserAgent->new( timeout => $self->{timeout} );
+
+	my $rqst = HTTP::Request::Common::POST(
+	    "$self->{proxy}/$service", \%arg );
+
+	$self->{trace} and print STDERR $rqst->as_string();
+	$rslt = $ua->request( $rqst );
+	$self->{trace} and print STDERR $rslt->as_string();
+    }
+
+    if ( _instance( $rslt, 'HTTP::Response' ) ) {
+
+	$rslt->is_success()
+	    or do {
+	    my $error = $rslt->status_line();
+	    chomp $error;
+	    die "$error\n";
+	};
+
+	$rslt = $rslt->content();
+
+	my $psr = $self->{_xml_parser} ||=
+	    XML::Parser->new( Style => 'Tree' );
+	$rslt = $psr->parse( $rslt );
+	$rslt = $self->_transform_xml( $rslt );
+
+    }
+
+    return $rslt;
+}
+
+#	my $resp = $self->_transform_xml( $xml );
+#
+#	This method takes as input the XML::Parser parse tree of the XML
+#	returned in response to an HTTP request to the USGS' elevation
+#	service, and transforms it, returning a reference to a hash that
+#	is supposed to be compatible with the hash returned by the
+#	corresponding SOAP::Lite request.
+#
+#	If a tree node has children other than literals, it calls itself
+#	recursively to parse them, and returns a hash keyed by the names
+#	of the children, and whose contents are the return generated by
+#	the recursive call. If a node has no children other than
+#	literals, it returns the concatenation of all non-white-space
+#	literals in the node.
+
+sub _transform_xml {
+    my ( $self, $parse ) = @_;
+    my @expand = @{ $parse };
+    my %rslt;
+    'HASH' eq ref $expand[0]
+	and shift @expand;
+    my @literal;
+    while ( @expand ) {
+	my ( $name, $content ) = splice @expand, 0, 2;
+	if ( ref $content ) {
+	    if ( exists $rslt{$name} ) {
+		ref $rslt{$name}
+		    or $rslt{$name} = [ $rslt{$name} ];
+		push @{ $rslt{$name} }, $self->_transform_xml(
+		    $content );
+	    } else {
+		$rslt{$name} = $self->_transform_xml( $content );
+	    }
+	} elsif ( $content !~ m/ \A \s* \z /smx ) {
+	    push @literal, $content;
+	}
+    }
+    return %rslt ? \%rslt : join ' ', @literal;
 }
 
 
@@ -1128,7 +1182,7 @@ if the USGS changes the WSDL and this module has not been modified to
 track the change.
 
 The default is
-'http://gisdata.usgs.gov/xmlwebservices2/elevation_service.asmx'.
+C<'http://gisdata.usgs.gov/XMLWebServices2/Elevation_Service.asmx'>.
 
 =head3 retry (unsigned integer)
 
